/* eslint-disable import/no-dynamic-require, global-require */

const path = require('path');
const _ = require('lodash');
const bluebird = require('bluebird');
const fs = require('fs-extra');
const getConfig = require('../utils/getManifestConfig');
const download = require('../utils/download');
const manifest = require('../utils/manifest');
const generateCustomComponentsImport = require('../utils/generateCustomComponentsImport');
const { styleManifestAssetsFileName } = require('../config/settings');

const pathToManifestAssets = path.join(
  process.cwd(),
  getConfig().pathToComponents,
  styleManifestAssetsFileName
);
const PROMISE_OPTIONS = { concurrency: 5 };

const convertFileNameToPath = file =>
  path.join(process.cwd(), getConfig().pathToComponents, file);

const downloadStyleFile = (pathToComponent, component) => {
  const { dir } = path.parse(pathToComponent);
  fs.mkdirsSync(dir);
  return download(`${getConfig().host}/api/component-file?component=${component}`, pathToComponent);
};

const downloadGroupFiles = (groupFiles, components) =>
  bluebird.map(groupFiles, (pathToComponent, i) =>
    downloadStyleFile(pathToComponent, components[i])
      .then(() => console.info(`Файл ${components[i]} был загружен`))
  , PROMISE_OPTIONS);


const downloadAllStyleFiles = () => {
  const components = _.keys(manifest.show());
  const assetComponents = _.chain(manifest.showAssets()).values().flatten().value();
  const pathsToComponents = components.map(convertFileNameToPath);
  const pathsToAssets = assetComponents.map(convertFileNameToPath);

  const downloadedComponents = downloadGroupFiles(pathsToComponents, components);
  const downloadedAssets = downloadGroupFiles(pathsToAssets, assetComponents);

  return Promise.all([downloadedComponents, downloadedAssets]);
};

const action = (cmd) => {
  if (!cmd.all && !cmd.component) {
    console.info('Нужно указать компонент');
    process.exit(1);
  }
  if (cmd.all) {
    manifest
      .download(getConfig().components)
      .then(() => {
        if (getConfig().components.length) {
          generateCustomComponentsImport();
        }
        downloadAllStyleFiles().catch((err) => {
          console.error('Не все файлы были загружены', err);
        });
      }).catch(console.error);
    return;
  }

  if (cmd.component) {
    manifest
      .update(cmd.component)
      .then(() => {
        const pathToComponent = convertFileNameToPath(cmd.component);
        const componentName = path.parse(cmd.component).dir.split(path.sep)[0];
        const assetComponents = require(pathToManifestAssets)[componentName];
        let pathsToAssets = [];

        if (_.isArray(assetComponents) && assetComponents.length !== 0) {
          pathsToAssets = assetComponents.map(convertFileNameToPath);
        }

        Promise.all([
          downloadStyleFile(pathToComponent, cmd.component),
          downloadGroupFiles(pathsToAssets, assetComponents)
        ]).then(() => {
          if (getConfig().components.length) {
            generateCustomComponentsImport();
          }
        }).catch((err) => {
          console.error(err);
        });
      });
  }
};

module.exports = action;
