#!/usr/bin/env node
/* eslint-disable import/no-dynamic-require, global-require */

const { vesion } = require('../config/settings');
const {
  setup,
  update,
  check,
  addComponents,
  removeComponents } = require('../actions');

let commander;

try {
  commander = require('commander');

  commander
    .version(vesion);

  commander
    .command('setup <path> <host> [components...]')
    .description('Установка начальных параметров')
    .action(setup);

  commander
    .command('update')
    .description('Обновление файлов стилей')
    .option('--all', 'Обновить все стили')
    .option('-c, --component <component>', 'Обновить отдельный компонент стилей')
    .action(update);

  commander
    .command('check')
    .description('Проверить обновления для стилей')
    .action(check);

  commander
    .command('add-component <component>')
    .description('Добавление компонента')
    .action(addComponents);

  commander
    .command('remove-component <component>')
    .description('Удаление компонента')
    .action(removeComponents);

  commander.parse(process.argv);
} catch (e) {
  if (e instanceof Error && e.code === 'MODULE_NOT_FOUND') {
    console.error('e', e);
    console.error('npm package commander не установлен, выполните `yarn add commander`');
  } else {
    console.error(e);
  }
}
