const fs = require('fs-extra');
const path = require('path');
const fetch = require('node-fetch');
const _ = require('lodash');
const downloadUtil = require('./download');
const getConfig = require('./getManifestConfig');
const { getPathToStyleManifest, getPathToStyleManifestAssets } = require('../config/manifestPaths');
const { requiredStylesheetFiles } = require('../config/settings');

const apiManifestUrl = '/api/manifest';
const apiManifestAssetsUrl = '/api/manifest-assets';
const getManifestUrl = () => `${getConfig().host}${apiManifestUrl}`;
const getManifestAssetsUrl = () => `${getConfig().host}${apiManifestAssetsUrl}`;

const convertAssetComponentName = component =>
  _.compact(component.split(path.sep))[0];

const show = () =>
  fs.readJsonSync(getPathToStyleManifest());

const showAssets = () =>
  fs.readJsonSync(getPathToStyleManifestAssets());

const getRemoteManifest = () =>
  fetch(getManifestUrl()).then(res => res.json());

const getRemoteManifestAssets = () =>
  fetch(getManifestAssetsUrl()).then(res => res.json());

const download = (components) => {
  if (components.length) {
    const componentsWithRequireStyles = [...requiredStylesheetFiles, ...components];
    return Promise.all([
      getRemoteManifest(),
      getRemoteManifestAssets(),
    ]).then(([remoteManifest, remoteManifestAssets]) => {
      const componentNames = componentsWithRequireStyles.map(component =>
        path.parse(component).dir.split(path.sep)[0]
      );
      const customManifestComponents = _.pick(remoteManifest, componentsWithRequireStyles);
      const customManifestAssetsComponents = _.pick(remoteManifestAssets, componentNames);

      fs.writeJsonSync(getPathToStyleManifest(), customManifestComponents);
      fs.writeJsonSync(getPathToStyleManifestAssets(), customManifestAssetsComponents);

      console.info('Загрузка манифестов завершена');
    });
  }
  return Promise.all([
    downloadUtil(`${getConfig().host}${apiManifestUrl}`, getPathToStyleManifest()),
    downloadUtil(`${getConfig().host}${apiManifestAssetsUrl}`, getPathToStyleManifestAssets()),
  ]).then(() => {
    console.info('Загрузка манифестов завершена');
  }).catch((err) => {
    console.error(err);
  });
};

const update = component =>
  Promise.all([
    fetch(`${getManifestUrl()}?component=${component}`).then(res => res.json()),
    fetch(`${getManifestAssetsUrl()}?component=${convertAssetComponentName(component)}`).then(res => res.json()),
  ]).then(([manifestJson, manifestAssetsUrlJson]) => {
    const currentStyleManifest = show();
    const currentStyleAssetsManifest = showAssets();
    const newStyleManifest = Object.assign({}, currentStyleManifest, manifestJson);
    const newStyleAssetsManifest = Object.assign(
      {},
      currentStyleAssetsManifest,
      manifestAssetsUrlJson
    );

    fs.writeJsonSync(getPathToStyleManifest(), newStyleManifest);
    fs.writeJsonSync(getPathToStyleManifestAssets(), newStyleAssetsManifest);
    console.info(`Компонент \`${component}\` успешно обновлен`);
  });

const remove = (component) => {
  const assetComponent = path.parse(component).dir;

  return Promise.all([
    fs.writeJsonSync(getPathToStyleManifest(), _.omit(show(), [component])),
    fs.writeJsonSync(getPathToStyleManifestAssets(), _.omit(showAssets(), [assetComponent]))
  ]);
};

module.exports = {
  getRemoteManifest,
  getRemoteManifestAssets,
  download,
  update,
  remove,
  show,
  showAssets,
};
